<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Http;

class SyncController extends Controller
{
    protected $pipeDriveController;
    protected $evoTalksController;
    protected $tabelaPreco;

    public function __construct(PipeDriveController $pipeDriveController, EvoTalksController $evoTalksController, TabelaPreco $tabelaPreco)
    {
        $this->pipeDriveController = $pipeDriveController;
        $this->evoTalksController = $evoTalksController;
        $this->tabelaPreco = $tabelaPreco;
    }

    public function syncCreatedOpportunityFromPipeDrive(Request $request)
    {
        Log::info('Recebendo webhook do Pipedrive', ['payload' => $request->all()]);

        $pipeDriveData = $request->input('data');
        //$pipeDriveData = $request->all();
        Log::info('ID oportunidade: ', [$pipeDriveData['id']]);
        $data = $this->pipeDriveController->getDeal($pipeDriveData['id']);
        $dealV2 = $this->pipeDriveController->getDealV2($pipeDriveData['id']);

        if (!$data) {
            Log::error("Falha ao recuperar dados do negócio com ID {$pipeDriveData['id']}.");
            return response()->json(['error' => 'Failed to retrieve deal data'], 500);
        }

        if (!$this->isValidPayload($data) || ($data['status'] ?? '') !== 'won') {
            Log::warning('Payload inválido ou oportunidade não marcada como "ganha"', ['payload' => $data]);
            return response()->json(['error' => 'Invalid or non-won opportunity'], 400);
        }

        $fkPipeline = $this->getPipelineId($data['pipeline_id'], $dealV2['label_ids'] == [] ? [] : $dealV2['label_ids']);
        if (!$fkPipeline) {
            Log::warning('Pipeline não permitido', ['pipeline_id' => $data['pipeline_id']]);
            return response()->json(['error' => 'Pipeline not allowed'], 403);
        }

        $fkStage = $this->getStageId($fkPipeline, $data['stage_id']);
        $contact = $this->syncPipeDriveContactToEvoTalks($data);
        $evoData = $this->mapPipeDriveToEvoTalks($data, $dealV2, $fkPipeline, $fkStage, $contact);
        $response = $this->evoTalksController->createOpportunity($evoData);
        Log::info('Mapeando dados para EvoTalks', ['pipeline_id' => $data['pipeline_id'], 'stage_id' => $fkStage]);
        Log::info('Enviando dados para criação de oportunidade no EvoTalks', ['evoData' => $evoData->all()]);

        if ($response->status() === 200 && isset($response->getData()->id)) {

            $opportunityId = $response->getData()->id;
            $notes = $this->pipeDriveController->fetchNotes($data['id']);
            
            if ($notes) {
                Log::info("Adicionando notas à oportunidade no EvoTalks", ['opportunity_id' => $opportunityId, 'notes' => $notes]);

                $this->evoTalksController->addNotesToOpportunity(new Request([
                    'opportunity_id' => $opportunityId,
                    'notes' => $notes,
                ]));
            }

            $this->syncActivitiesFromPipeDrive(new Request(['deal_id' => $data['id']]), $opportunityId);
        } else {
            Log::error('Erro ao criar oportunidade no EvoTalks', ['response' => $response->json()]);
        }

        return $response;
    }

    public function syncActivitiesFromPipeDrive(Request $request, $opportunityId)
    {
        $dealId = $request->input('deal_id');
        Log::info('Iniciando sincronização de atividades do Pipedrive para o EvoTalks');

        if (!$dealId) {
            Log::warning('ID do negócio (deal_id) não fornecido');
            return response()->json(['error' => 'deal_id is required'], 400);
        }

        $activities = $this->pipeDriveController->fetchDealFlowActivities($dealId);

        if (empty($activities)) {
            Log::info('Nenhuma atividade do tipo "activity" encontrada no Pipedrive', ['deal_id' => $dealId]);
            return response()->json(['message' => 'No activities found for the given deal'], 204);
        }

        foreach ($activities as $activity) {
            $this->syncPipeDriveActivitiesToEvoTalks($activity, $opportunityId);
        }

        return response()->json(['message' => 'Activities synced successfully'], 200);
    }

    private function isValidPayload($data)
    {
        return is_array($data) && isset($data['id'], $data['status'], $data['pipeline_id']);
    }

    function mapPipeDriveTypeToEvotalksAction($pipedriveType)
    {
        $typeMapping = [
            'call'      => 1,       // Ligar
            'meeting'   => 3,       // Reunião
            'chat'      => 2,       // Chat
            'whatsapp'  => 2,       // Chat (WhatsApp)
            'task'      => 0,       // Sem ação
        ];

        return $typeMapping[$pipedriveType] ?? 0;
    }

    private function mapProducts($products)
    {
        $mappedProducts = [];

        if (!is_array($products)) {
            return $mappedProducts;
        }

        $mappedProducts[0] = [
            'id'        => $this->tabelaPreco->getTaxaOperacao($products['products_count'])['id'],
            'qty'       => $products['products_count'],
            'discount'  => 0,
        ];

        $mappedProducts[1] = [
            'id'        => 2,
            'qty'       => 0,
            'discount'  => 0,
        ];

        Log::info('Produtos mapeados: ', $mappedProducts);

        return $mappedProducts;
    }

    private function mapTags($label_ids)
    {
        $mappedTags = [
            247 => 2,
            248 => 3,
            249 => 4
        ];

        $mappedResult = array_values(array_map(function ($id) use ($mappedTags) {
            return $mappedTags[$id] ?? null;
        }, array_filter($label_ids, function ($id) use ($mappedTags) {
            return isset($mappedTags[$id]);
        })));

        if(!in_array(9, $mappedResult, true)) {
            $mappedResult[] = 9;
        }

        return $mappedResult;
    }

    private function mapPipeDriveToEvoTalks($pipeDriveData, $dealV2, $fkPipeline, $fkStage, $contact)
    {

        $value = $pipeDriveData['value'] ?? 0;
        $productsCount = (!isset($pipeDriveData['products_count']) || $pipeDriveData['products_count'] == 0) ? 1 : $pipeDriveData['products_count'];
        $FeeValue = $value == 0 ? 0 : number_format($value / $productsCount, 2, '.', '');

        return new Request([
            'fkPipeline'        => $fkPipeline,
            'fkStage'           => $fkStage,
            'responsableid'     => 507,
            'title'             => $pipeDriveData['title'] ?? '',
            'clientid'          => $this->safeGet($pipeDriveData, 'person_id.name'),
            'mainphone'         => $this->safeGet($pipeDriveData, 'person_id.phone.0.value'),
            'mainmail'          => $pipeDriveData['person_id']['email'][0]['value'],
            'description'       => preg_replace('/<br\s*\/?>/i', "\n", $this->safeGet($dealV2, 'custom_fields.8ebbdb58a2067decc8b6d2d0849bb67c4bfc13d1')),
            'expectedclosedate' => $pipeDriveData['expected_close_date'] ?? null,
            'formattedlocation' => $this->safeGet($pipeDriveData, 'org_id.name'),
            'postalcode'        => $this->safeGet($pipeDriveData, 'org_id.address_postal_code', ''),
            'address1'          => $this->safeGet($pipeDriveData, 'org_id.address', ''),
            'address2'          => '',
            'city'              => $this->safeGet($pipeDriveData, 'org_id.address_city', ''),
            'state'             => $this->safeGet($pipeDriveData, 'org_id.address_state', ''),
            'country'           => $this->safeGet($pipeDriveData, 'org_id.address_country', ''),
            'countrycode'       => $this->safeGet($pipeDriveData, 'org_id.address_country_code', ''),
            'lat'               => $this->safeGet($pipeDriveData, 'org_id.address_lat', 0),
            'lon'               => $this->safeGet($pipeDriveData, 'org_id.address_long', 0),
            'probability'       => $pipeDriveData['probability'] ?? 0,
            'value'             => $FeeValue,
            'recurrentvalue'    => $pipeDriveData['recurrentvalue'] ?? 0,
            'origin'            => 0,
            'formsdata'         => (object) [],
            'tags'              => $dealV2['label_ids'] == [] ? [] : $this->mapTags($dealV2['label_ids']),
            'files'             => [],
            'contacts'          => [$contact],
            'followers'         => [],
            'products'          => $value == 0 || $pipeDriveData['products_count'] == 0 ? [] : $this->mapProducts($pipeDriveData ?? []),
        ]);
    }

    public function syncPipeDriveActivitiesToEvoTalks(array $activity, int $opportunityId)
    {
        if (!isset($activity['data']['type'])) {
            Log::warning('Atividade sem tipo definido', ['activity' => $activity['data']['type']]);
            return;
        }

        $data = [
            'owner'         => 507,
            'title'         => $activity['data']['subject'] ?? 'Sem título',
            'description'   => $activity['data']['note'] ?? 'Sem descrição',
            'duedate'       => $activity['data']['due_date'] ?? null,
            'fkOpportunity' => $opportunityId,
            'progress'      => 0,
            'action'        => $this->mapPipeDriveTypeToEvotalksAction($activity['data']['type']) ?? 0,
            'actiondata'    => '',
            'checklist'     => [],
            'watchers'      => [],
            'contacts'      => [],
            'users'         => [1],
            'files'         => [],
            'tags'          => [],
        ];

        try {
            $taskResponse = $this->evoTalksController->createTask(new Request($data));
            Log::info('Resposta da criação da tarefa', ['taskResponse' => $taskResponse]);

            if (isset($activity['data']['done']) && $activity['data']['done'] === true) {
                $this->evoTalksController->markAsDone($taskResponse['id'], $activity);
            }
        } catch (\Exception $e) {
            Log::error('Erro ao sincronizar atividade com EvoTalks', ['error' => $e->getMessage()]);
        }
    }

    public function syncPipeDriveContactToEvoTalks($validated)
    {
        $contactData = [
            'name'          => $validated['person_id']['name'],
            'document'      => '',
            'number'        => $validated['person_id']['phone'][0]['value'] ?? null,
            'email'         => $validated['person_id']['email'][0]['value'] ?? null,
            'address'       => $validated['org_id']['address'] ?? null,
            'tag'           => [1]
        ];

        $contactResponse = $this->evoTalksController->addContact($contactData);

        return $contactResponse['contactId'];

        if (!$contactResponse) {
            return response()->json(['error' => 'Erro ao cadastrar contato no EvoTalks'], 500);
        }
    }

    private function getPipelineId($pipelineId, $tag)
    {
        $selfServiceTag = in_array(251, $tag);

        if ($pipelineId == 1) {
            return 1; // Pipeline "Vendas"
        } elseif ($pipelineId == 3) {
            return 1; // Pipeline "CS - Onboarding"
        } elseif ($pipelineId == 9 || $selfServiceTag) {
            return 3; // Pipeline "Vendas Self-service"
        } elseif ($pipelineId == 4) {
            return 2; // Pipeline "CS - Ongoing"
        }
        return null;
    }

    private function getStageId($pipelineId, $stage_id)
    {
        if ($pipelineId == 1) {
            return 1; // Stage para Pipeline "Vendas"
        } elseif ($pipelineId == 3) {
            return 16; // Stage para Pipeline "Vendas Self-service"
        } elseif ($pipelineId == 2) {
            if ($stage_id == 29 || $stage_id == 27) {
                return 11; // Stage para Pipeline "Vendas Self-service - Pos Onboarding"
            } elseif ($stage_id == 54) {
                return 12; // Stage para Pipeline "Vendas Self-service - Abaixo de 80% de liquidação"
            } elseif ($stage_id == 31) {
                return 13; // Stage para Pipeline "Vendas Self-service - Acima de 80% de liquidação"
            } elseif ($stage_id == 32) {
                return 14; // Stage para Pipeline "Vendas Self-service - Em queda de liquidação"
            } elseif ($stage_id == 65) {
                return 15; // Stage para Pipeline "Vendas Self-service - Retenção"
            }
        }
        return null;
    }

    private function safeGet($array, $keyPath, $default = null)
    {
        $keys = explode('.', $keyPath);
        foreach ($keys as $key) {
            if (!is_array($array) || !isset($array[$key])) {
                return $default;
            }
            $array = $array[$key];
        }
        return $array;
    }
}
